@@ -1,7 +1,8 @@
-/* bmdrv.c
- * kernel driver for BLINKENmini
+/* kernel driver for BLINKENmini
  *  - character device kernel module for input from and output to BLINKENmini using the parallel port
- * Copyright (C) 2002 sphaera & 1stein (http://blinkenmini.1stein.no-ip.com/)
+ * version 1.1.92 date 2006-01-08
+ * Copyright (C) 2002-2006 sphaera & 1stein (http://blinkenmini.schuermans.info/)
+ * Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -23,7 +24,10 @@
 #include <linux/version.h>
 #include <linux/ioport.h>
 #include <linux/fs.h>
+#if LINUX_VERSION_CODE >= 0x20600 //kernel-version 2.6
+#else
 #include <linux/wrapper.h>
+#endif
 #include <asm/uaccess.h>
 #include <asm/io.h>
 #include <asm/param.h>
@@ -47,9 +51,10 @@ MODULE_PARM( irq, "i" );
 //global constants concerning the module
 #define BmDrvDevName "bmdrv" //the name of this device (max. 31 chars allowed)
 #define BmDrvDevId ((void *)(('b'<<24)+('m'<<16)+('d'<<8)+'r')) //the id of this device
+//#error new_ver: UPDATE numerical version information!!!
 #define BmDrvDevVerMaj 1 //version of this device
 #define BmDrvDevVerMin 1
-#define BmDrvDevVerRev 90
+#define BmDrvDevVerRev 92
 
 //global types
 #define BmDrvInternalPictureSizeX 18 //dimensions of an internal BM-picture
@@ -421,8 +426,10 @@ int BmDrvOutFileOpen( struct inode * Inode, struct file * File )
   //output device is now open
   BmDrvOutIsOpen = 1;
   BmDrvOutFile = File; //remember pointer to file-struct
+#if LINUX_VERSION_CODE < 0x20400 //not for kernel-version 2.4
   //module is now used once more
   MOD_INC_USE_COUNT;
+#endif
 
   //empty next picture that belongs not to the interrupt
   memset( &BmDrvNextPictures[1 - BmDrvNextPictureInterruptIndex], 0, sizeof( struct tBmDrvInternalPicture ) );
@@ -455,8 +462,10 @@ int BmDrvInFileOpen( struct inode * Inode, struct file * File )
   //input device is now open
   BmDrvInIsOpen = 1;
   BmDrvInFile = File; //remember pointer to file-struct
+#if LINUX_VERSION_CODE < 0x20400 //not for kernel-version 2.4
   //module is now used once more
   MOD_INC_USE_COUNT;
+#endif
 
   //open succeeded
   return 0;  
@@ -497,8 +506,10 @@ int BmDrvOutFileClose( struct inode * Inode, struct file * File )
   //output device is now closed
   BmDrvOutIsOpen = 0;
   BmDrvOutFile = NULL;
+#if LINUX_VERSION_CODE < 0x20400 //not for kernel-version 2.4
   //module is now used once less
   MOD_DEC_USE_COUNT;
+#endif
 
   //close succeeded
   return 0;  
@@ -515,8 +526,10 @@ int BmDrvInFileClose( struct inode * Inode, struct file * File )
   //input device is now closed
   BmDrvInIsOpen = 0;
   BmDrvInFile = NULL;
+#if LINUX_VERSION_CODE < 0x20400 //not for kernel-version 2.4
   //module is now used once less
   MOD_DEC_USE_COUNT;
+#endif
 
   //close succeeded
   return 0;  
@@ -545,7 +558,7 @@ int BmDrvIoctlGetVersion( struct tBmDrvVersion * pBmDrvVersion )
   struct tBmDrvVersion Ver;
  
   //test pointer
-  if( verify_area( VERIFY_WRITE, pBmDrvVersion, sizeof( struct tBmDrvVersion ) ) != 0 )
+  if( !access_ok( VERIFY_WRITE, pBmDrvVersion, sizeof( struct tBmDrvVersion ) ) )
     //pointer not okay
     return -EFAULT;
 
@@ -569,7 +582,7 @@ int BmDrvIoctlGetPictureSize( struct tBmDrvPictureSize * pBmDrvPictureSize )
   struct tBmDrvPictureSize Size;
  
   //test pointer
-  if( verify_area( VERIFY_WRITE, pBmDrvPictureSize, sizeof( struct tBmDrvPictureSize ) ) != 0 )
+  if( !access_ok( VERIFY_WRITE, pBmDrvPictureSize, sizeof( struct tBmDrvPictureSize ) ) )
     //pointer not okay
     return -EFAULT;
 
@@ -591,7 +604,7 @@ int BmDrvIoctlSetPicture( struct tBmDrvPicture * pBmDrvPicture )
   struct tBmDrvPicture Picture;
 
   //test pointer
-  if( verify_area( VERIFY_READ, pBmDrvPicture, sizeof( struct tBmDrvPicture ) ) != 0 )
+  if( !access_ok( VERIFY_READ, pBmDrvPicture, sizeof( struct tBmDrvPicture ) ) )
     //pointer not okay
     return -EFAULT;
 
@@ -609,7 +622,7 @@ int BmDrvIoctlSetPicture( struct tBmDrvPicture * pBmDrvPicture )
   //  SizeX * SizeY == sizeof( struct tBmDrvInternalPicture )
 
   //test pixel-buffer
-  if( verify_area( VERIFY_READ, Picture.pPixels, sizeof( struct tBmDrvInternalPicture ) ) != 0 )
+  if( !access_ok( VERIFY_READ, Picture.pPixels, sizeof( struct tBmDrvInternalPicture ) ) )
     //buffer is invalid
     return BmDrvErrPixelBuffer;
 
@@ -633,7 +646,7 @@ int BmDrvIoctlGetButtonInfo( struct tBmDrvButtonInfo * pBmDrvButtonInfo )
   struct tBmDrvButtonInfo Info;
  
   //test pointer
-  if( verify_area( VERIFY_WRITE, pBmDrvButtonInfo, sizeof( struct tBmDrvButtonInfo ) ) != 0 )
+  if( !access_ok( VERIFY_WRITE, pBmDrvButtonInfo, sizeof( struct tBmDrvButtonInfo ) ) )
     //pointer not okay
     return -EFAULT;
 
@@ -657,7 +670,7 @@ int BmDrvIoctlGetButtonState( struct tBmDrvButtonState * pBmDrvButtonState )
   struct tBmDrvInternalControllerButtons ButtonState;
  
   //test pointer
-  if( verify_area( VERIFY_READ, pBmDrvButtonState, sizeof( struct tBmDrvButtonState ) ) != 0 )
+  if( !access_ok( VERIFY_READ, pBmDrvButtonState, sizeof( struct tBmDrvButtonState ) ) )
     //pointer not okay
     return -EFAULT;
 
@@ -675,7 +688,7 @@ int BmDrvIoctlGetButtonState( struct tBmDrvButtonState * pBmDrvButtonState )
   //  ControllerCnt * ButtonCnt == sizeof( struct tBmDrvInternalControllerButtons )
 
   //test button-buffer
-  if( verify_area( VERIFY_WRITE, State.pButtons, sizeof( struct tBmDrvInternalControllerButtons ) ) != 0 )
+  if( !access_ok( VERIFY_WRITE, State.pButtons, sizeof( struct tBmDrvInternalControllerButtons ) ) )
     //buffer is invalid
     return BmDrvErrBtnBuffer;
 
@@ -792,7 +805,7 @@ ssize_t BmDrvOutFileWrite( struct file * File, const char * BufferPtr, size_t Bu
     return -EBADF;
 
   //test buffer
-  if( verify_area( VERIFY_READ, BufferPtr, BufferLen ) != 0 )
+  if( !access_ok( VERIFY_READ, BufferPtr, BufferLen ) )
     //buffer not okay
     return -EFAULT;
 
@@ -895,7 +908,7 @@ ssize_t BmDrvInFileWrite( struct file * File, const char * BufferPtr, size_t Buf
     return -EBADF;
 
   //test buffer
-  if( verify_area( VERIFY_READ, BufferPtr, BufferLen ) != 0 )
+  if( !access_ok( VERIFY_READ, BufferPtr, BufferLen ) )
     //buffer not okay
     return -EFAULT;
 
@@ -928,7 +941,7 @@ ssize_t BmDrvOutFileRead( struct file * File, char * BufferPtr, size_t BufferLen
     return -EBADF;
 
   //test buffer
-  if( verify_area( VERIFY_WRITE, BufferPtr, BufferLen ) != 0 )
+  if( !access_ok( VERIFY_WRITE, BufferPtr, BufferLen ) )
     //buffer not okay
     return -EFAULT;
 
@@ -950,7 +963,7 @@ ssize_t BmDrvInFileRead( struct file * File, char * BufferPtr, size_t BufferLen,
     return -EBADF;
 
   //test buffer
-  if( verify_area( VERIFY_WRITE, BufferPtr, BufferLen ) != 0 )
+  if( !access_ok( VERIFY_WRITE, BufferPtr, BufferLen ) )
     //buffer not okay
     return -EFAULT;
 
@@ -1029,7 +1042,7 @@ ssize_t BmDrvFileRead( struct file * File, char * BufferPtr, size_t BufferLen, l
 struct file_operations BmDrvFileOperations =
 {
 #if LINUX_VERSION_CODE >= 0x20400 //kernel-version 2.4
-  owner: NULL,
+  owner: THIS_MODULE,
 #endif
   llseek: NULL,
   read: BmDrvFileRead,
@@ -1220,10 +1233,9 @@ void cleanup_module( )
   //unregister io-ports
   release_region( BmDrvIoAddr, 3 );
 
-    //unregister device
+  //unregister device
   unregister_chrdev( BmDrvDevNoMajor, BmDrvDevName );
 
   //done
   printk( "bmdrv: unloaded...\n" );
 }
-
